@@ -1,5 +1,7 @@
 from agox.models.descriptors.ABC_descriptor import DescriptorBaseClass
 from agox.models.descriptors.fingerprint import Fingerprint
-from agox.models.descriptors.spectral_graph_descriptor import SpectralGraphDescriptor
-from agox.models.descriptors.soap import SOAP
 from agox.models.descriptors.simple_fingerprint import SimpleFingerprint
+from agox.models.descriptors.soap import SOAP
+from agox.models.descriptors.spectral_graph_descriptor import SpectralGraphDescriptor
+from agox.models.descriptors.voronoi import Voronoi
+from agox.models.descriptors.voronoi_site import VoronoiSite
