@@ -1,30 +1,30 @@
-import pytest
-from ase.io import read, write
-from agox.environments import Environment
-from agox.candidates import StandardCandidate
-from ase import Atoms
 import numpy as np
+import pytest
+from ase.io import read
 
+from agox.candidates import StandardCandidate
+from agox.environments import Environment
 from agox.test.test_utils import test_data_dicts
 
+
 @pytest.fixture(params=test_data_dicts)
 def environment_and_dataset(request):
-    atoms = read(request.param['path'])
+    atoms = read(request.param["path"])
     cell = atoms.get_cell()
     corner = np.array([0, 0, 0])
-    remove = request.param['remove']
-    numbers = atoms.get_atomic_numbers()[len(atoms)-remove:]
+    remove = request.param["remove"]
+    numbers = atoms.get_atomic_numbers()[len(atoms) - remove :]
 
-    template = read(request.param['path'])
-    del template[len(template)-remove:len(template)]
-    environment = Environment(template=template, numbers=numbers, confinement_cell=cell, 
-            confinement_corner=corner)
+    template = read(request.param["path"])
+    del template[len(template) - remove : len(template)]
+    environment = Environment(template=template, numbers=numbers, confinement_cell=cell, confinement_corner=corner)
 
-    data = read(request.param['path'], ':')
+    data = read(request.param["path"], ":")
     candidates = [StandardCandidate.from_atoms(template, a) for a in data]
 
     return environment, candidates
 
+
 def test_environment(environment_and_dataset):
     environment, dataset = environment_and_dataset
 
@@ -38,8 +38,3 @@ def test_environment(environment_and_dataset):
 
     missing = environment.get_missing_types()
     assert isinstance(missing, np.ndarray)
-
-
-
-
-
